/*
 * Compilatori/preprocessori consigliati:
 * Linux:   g++
 * Windows: MinGW
*/

#ifdef __linux__
  #define os 1
#elif _WIN32
  #define os 2
  #include <ctime>
#else
  #define os 0
#endif

#define DEBUG 1 /* 0 disabled, other enabled */

#include <iostream>
using namespace std;
#include <fstream>
#include <SDL/SDL.h>
#include <SDL/SDL_gfxPrimitives.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_mixer.h>
#include <SDL/SDL_ttf.h>

#include "tipi.cpp"
#include "declare.cpp"
#include "funzioni.cpp"

/* Messaggi di debug */
void debug(const char * stringa)
{ if (DEBUG) clog << "debug: " << stringa << endl; }

char versione[15]; /* Numero di versione del programma */
char ROOTDIR[256]; /* Cartella in cui si trovano i file di configurazione di ppong */
SDL_Surface* schermo = NULL;
Uint8 *tasto = SDL_GetKeyState(NULL); /* Controlla keyinput da tastiera */
SDL_Event evento;
double lastframe = time(NULL); /* vedere fps() */
/* Numero di blocchi nello schermo, in larghezza e altezza */
const unsigned short width = 16;
const unsigned short height = 24;
/* Dimensioni fisiche dei blocchi, in pixel */
const unsigned short largBlock = 640 / width;
const unsigned short altBlock = 480 / height;
#define N_ICONE 14
SDL_Surface *icona[N_ICONE];
#define N_TEXTURES 9
SDL_Surface *texture[N_TEXTURES]; /* motivi sui mattoncini */
opt opzione;
#define N_MUSIC 2
#define N_EFFECTS 6
Mix_Music *musica[N_MUSIC];
Mix_Chunk *effetto[N_EFFECTS];
unsigned int lvl; /* Livello corrente */
fstream filelvl; /* File con gli schemi per i livelli - vedere livelli.cpp per la descrizione */
char title[64]; /* Titolo-nome del livello */
#define FONTW 9 /* non utilizzato per caratteri TTF */
#define FONTH 15
TTF_Font *carattere = NULL;
#define MENU_SOUND 0
#define MENU_ERROR_SOUND 2
/* Opzioni varie */
Uint32 SCHERMOINTERO = 0;
int SCREENWIDTH = 854;
int DEPTH = 32;

#include "player.cpp"
#include "block.cpp"
brick block[width][height];
#include "classi.cpp"

#include "grafica.cpp"
#include "salva.cpp"
#include "menu.cpp"
#include "livelli.cpp"
#include "gameover.cpp"
#include "gioca.cpp"
#include "opzioni.cpp"
#include "info.cpp"

/* Chiude tutto liberando memoria e
 * rilasciando i livelli di astrazione SDL in maniera sicura */
void halt()
{
  Mix_Quit();
  IMG_Quit();
  TTF_CloseFont(carattere);
  TTF_Quit();
  SDL_Quit();
}

int main(int argc, char *argv[])
{
while (true)
{
  /* NUMERO DI VERSIONE DEL PROGRAMMA */
  strcpy(versione, "0.99.2");

  /* Imposta variabili d'ambiente e altro a seconda del sistema operativo
   * vedere le direttive del preprocessore all'inizio di questo file */
  switch (os)
  {
    case 1:
      strcpy(ROOTDIR, getenv("HOME"));
      strcat(ROOTDIR, "/.ppong/");
      cout << "[I] Linux operating system detected\n" << endl;
      break;
    case 2:
      strcpy(ROOTDIR, getenv("USERPROFILE"));
      strcat(ROOTDIR, "\\ppong\\");
      clog << "[W] Windows operating system detected. Some features may not work!\n";
      break;
    default: cerr << "[E] Unable to detect operating system: some features may not be available!\n"; break;
  }

  /* azzera i puntatori delle immagini, della musica e degli effetti sonori*/
  for (short n = 0; n < N_ICONE; n++) icona[n] = NULL;
  for (short n = 0; n < N_TEXTURES; n++) texture[n] = NULL;
  for (short n = 0; n < N_MUSIC; n++) musica[n] = NULL;
  for (short n = 0; n < N_EFFECTS; n++) effetto[n] = NULL;

  srand(time(NULL));
  /* Impostazione delle opzioni di default */
  opzione.difficolta = 3; /* Valori 1-6 in ordine crescente */
  opzione.startLvl = 0;
  opzione.sonoro = true;
  opzione.autosave = false;
  opzione.volumeEffetti = 60;
  opzione.volumeMusica = 90;
  opzione.mouse = true;
  opzione.panscan = false;
  opzione.maxfps = 33; /* ~30 fps*/
  opzione.fullscreen = false;
  opzione.lq = false;
  opzione.grabmouse = false;
  /* Impostazione delle opzioni salvate su file */
  if (! caricaOpzioni()) clog << "[W] Unable to load settings from file!\n";
  /* Impostazione delle opzioni passate da riga di comando */
  for (short v = 0; v < argc; v++)
  {
    if (argv[v][0] == '-')
    {
      switch (argv[v][1])
      {
        case 'l': opzione.startLvl = atoi(argv[v + 1]) - 1; break;
        case 'd': opzione.difficolta = atoi(argv[v + 1]); break;
        //case 'r': opzione.maxfps = 1000 / atoi(argv[v + 1]); break;
        case 'm': opzione.sonoro = false; break;
        case 'c': opzione.panscan = true; break;
        case 'h': infocmd(); return 0; break;
        case 'f': opzione.fullscreen = true; break;
        case 'w': opzione.grabmouse = true; break;
        case 'q': opzione.lq = true; break;
        case '1': DEPTH = 8; break;
        case '2': DEPTH = 16; break;
        case '3': DEPTH = 24; break;
        case '4': DEPTH = 32; break;
        case 'v': cout << "P Pong version " << versione << endl; return 0; break;
        default: clog << "[W] Parametro sconosciuto!\n"; break;
      }
    }
  }
  infolog();
  SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_TIMER); /* inizializzazione */

  /* Imposta titolo */
  SDL_WM_SetCaption("P Pong", "PPong");

  /* prepara lo schermo */
  if (opzione.fullscreen) SCHERMOINTERO = SDL_FULLSCREEN;
  if (opzione.panscan) SCREENWIDTH = 640;
  schermo = SDL_SetVideoMode(SCREENWIDTH, 480, DEPTH, SDL_HWSURFACE | SDL_DOUBLEBUF | SCHERMOINTERO);
  if (schermo == NULL)
  { cerr << "[E] Fatal error: Unable to SetVideoMode(). Run in a graphic environment!\n"; halt(); return 1; }

  if (! TTF_WasInit())
    if (TTF_Init() == -1) clog << "[W] Unable to TTF_Init()!" << endl;

  if (! caricafont()) msg("Fonts not loaded properly! Check stderr");

  if (IMG_Init(IMG_INIT_PNG) != IMG_INIT_PNG) { msg("FATAL ERROR: Unable to IMG_Init(): check stderr\n"); cerr << "[E] Unable to IMG_Init()!\n"; halt(); return 1; }
  if (! caricaIcone()) { msg("FATAL ERROR: Unable to caricaIcone(): check stderr"); cerr << "[E] Unable to caricaIcone()!\n"; halt(); return 1; }

  SDL_WM_SetIcon(icona[0], NULL);

  if (opzione.sonoro)
  {
    if (Mix_OpenAudio(MIX_DEFAULT_FREQUENCY, MIX_DEFAULT_FORMAT, 2, 1024) == -1)
    { clog << "[W] Unable to Mix_OpenAudio()\n"; } /* Caricamento dei file audio musicali principali */
    musica[0] = Mix_LoadMUS("data/intro.ogg");
    musica[1] = Mix_LoadMUS("data/background1.ogg");
    //~ musica[2] = Mix_LoadMUS("data/background2.ogg");
    for (short n = 0; n < N_MUSIC; n++) if (musica[n] == NULL) clog << "[W] Unable to Mix_LoadMUS() music #" << n << endl;

    Mix_VolumeMusic(opzione.volumeMusica);
    if (Mix_PlayingMusic()) Mix_HaltMusic();
      Mix_PlayMusic(musica[0], 0);

    if (! Mix_AllocateChannels(8)) clog << "[W] Unable to Mix_AllocateChannels()\n";
    effetto[0] = Mix_LoadWAV("data/sound00.wav");
    effetto[1] = Mix_LoadWAV("data/sound01.wav");
    effetto[2] = Mix_LoadWAV("data/sound02.wav");
    effetto[3] = Mix_LoadWAV("data/sound03.wav");
    effetto[4] = Mix_LoadWAV("data/sound04.wav");
    effetto[5] = Mix_LoadWAV("data/sound05.wav");
    for (short n = 0; n < N_EFFECTS; n++) if (effetto[n] == NULL) clog << "[W] Unable to Mix_LoadWAV() sound effect #" << n << endl;
  }
  else
  {
    cout << "[I] Sound disabled from commandline\n";
  }

  char r;
  do {
    r = ppong();
    if (r == 'q') { cout << "[I] Have a nice day...\n"; halt(); return 0; }
  } while (r != 'o');

} /* while (true) */
  halt();
  clog << "[W] Closing in an unespected way..." << endl;
  return 0;
}

char ppong()
{
  while (true)
  {
    filelvl.close(); filelvl.open("livelli.lvl", ios::in | ios::out | ios::binary);
    switch (menuEntry())
    {
      case 'q': return 'q'; break;
      case 'i': info(); break;
      case 'g': if (! gioca(true)) msg("FATAL ERROR! New Game: check stderr and stdout!"); break;
      case 'c': if (! gioca(false)) msg("FATAL ERROR! Load Game: check stderr and stdout!"); break;
      case 'o': if (opzioni()) { /*halt(); */cout << "[I] Reloading..." << endl; return 'o'; } break;
      default: break;
    }
  }

  halt();
  return 0;
}

/*
 * P Pong
 * Programmato da Giovan Battista Rolandi
 * http://gbr.altervista.org
 * 2012 2013
 *
 * Questo programma è distribuito sotto la
 * Licenza Creative Commons Attribuzione Non Commerciale Condividi allo Stesso Modo 3.0 Italia
 * altrimenti abbreviata Licenza CC BY-NC-SA
*/
